package com.jz.common.jwt;


import java.util.Date;

import org.apache.commons.lang3.StringUtils;

import com.jz.common.constant.Constants;
import io.jsonwebtoken.Claims;
import io.jsonwebtoken.Jwts;
import io.jsonwebtoken.SignatureAlgorithm;
import lombok.extern.slf4j.Slf4j;

/**
 * Secure工具类
 */
@Slf4j
public class JwtUtil {

    /**
     * 过期时间，5分钟
     */
   

    /**
     * 秘钥
     */
    public static final String APPSECRET = "9ZzhijiekeySyrTegfae2NItSfk";

    /**
     * 生成jwt
     *
     * @param userInfo
     * @return
     */
    public static String geneJsonWebToken(JwtUserInfo userInfo) {
        long nowMillis = System.currentTimeMillis();
        long expMillis = nowMillis + Constants.ACCESS_EXPIRE;
        Date exp = new Date(expMillis);        
        if (userInfo == null || StringUtils.isEmpty(userInfo.getUserId())) {
            return null;
        }
        String token = Jwts.builder().setSubject(userInfo.getUserId())
                .claim("userId", userInfo.getUserId())
                .claim("userName", userInfo.getUserName())
                .claim("loginName", userInfo.getLoginName())
                .claim("system", userInfo.getSystem())
                .claim("browserName", userInfo.getBrowserName())
                .claim("isPassword", userInfo.getIsPassword())
                .setIssuedAt(new Date())
//                .setExpiration(exp)
                .signWith(SignatureAlgorithm.HS256, APPSECRET).compact();
        return token;
    }

    public static String geneJsonWebRefreshToken(JwtUserInfo userInfo) {
        long nowMillis = System.currentTimeMillis();
        long expMillis = nowMillis + Constants.REFRESH_EXPIRE;
        Date exp = new Date(expMillis);        
        if (userInfo == null || StringUtils.isEmpty(userInfo.getUserId())) {
            return null;
        }
        String token = Jwts.builder().setSubject(userInfo.getUserId())
                .claim("userId", userInfo.getUserId())
                .claim("userName", userInfo.getUserName())
                .claim("loginName", userInfo.getLoginName())
                .setIssuedAt(new Date())
                .setExpiration(exp)
                .signWith(SignatureAlgorithm.HS256, APPSECRET).compact();
        return token;
    }

    /**
     * 校验token
     *
     * @param token
     * @return
     */
    public static Claims checkJWT(String token) {

        try {
            final Claims claims = Jwts.parser().setSigningKey(APPSECRET).
                    parseClaimsJws(token).getBody();
            return claims;

        } catch (Exception e) {
        }
        return null;
    }
    /**
     * 
     * @Title: getUserIdByToken 
     * @Description: TODO(获取userId) 
     * @最后修改人: niulichang-eleven
     * @最后修改时间: 2020年6月11日 上午11:33:53
     * @param token
     * @return 对方法的参数进行描述
     * @return: String 返回类型
     * @throws
     */
    public static String getUserIdByToken(String token) {
        String userId = null;
        Claims claims = JwtUtil.checkJWT(token);
        if (claims != null) {
        	userId = (String) claims.get("userId");
        }
        return userId;
    }
    /**
     * 
     * @Title: getLoginNameByToken 
     * @Description: TODO(获取登录名称) 
     * @最后修改人: niulichang-eleven
     * @最后修改时间: 2020年6月11日 上午11:34:09
     * @param token
     * @return 对方法的参数进行描述
     * @return: String 返回类型
     * @throws
     */
    public static String getLoginNameByToken(String token) {
        String loginName = null;
        Claims claims = JwtUtil.checkJWT(token);
        if (claims != null) {
        	loginName = (String) claims.get("loginName");
        } else {
            try {
                throw new Exception("非法token");
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
        return loginName;
    }
    /**
     * 
     * @Title: getUserNameByToken 
     * @Description: TODO(获取用户名称) 
     * @最后修改人: niulichang-eleven
     * @最后修改时间: 2020年6月11日 上午11:34:58
     * @param token
     * @return 对方法的参数进行描述
     * @return: String 返回类型
     * @throws
     */
    public static String getUserNameByToken(String token) {
        String userName = null;
        Claims claims = JwtUtil.checkJWT(token);
        if (claims != null) {
        	userName = (String) claims.get("userName");
        } else {
            try {
                throw new Exception("非法token");
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
        return userName;
    }

    public static String getIsPasswordByToken(String token) {
        String isPassword = null;
        Claims claims = JwtUtil.checkJWT(token);
        if (claims != null) {
        	isPassword = (String) claims.get("isPassword");
        } else {
            try {
                throw new Exception("非法token");
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
        return isPassword;
    }
}
