package com.jz.dmp.modules.service.impl;


import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.jz.common.constant.JsonResult;
import com.jz.common.constant.ResultCode;
import com.jz.common.page.PageInfoResponse;
import com.jz.common.persistence.BaseService;
import com.jz.common.utils.realTime.DBUtil;
import com.jz.dmp.modules.controller.DataIntegration.bean.DataSourceNameListDto;
import com.jz.dmp.modules.controller.DataIntegration.bean.RealTimeSyncListDto;
import com.jz.dmp.modules.controller.DataIntegration.bean.RealTimeSyncListReq;
import com.jz.dmp.modules.controller.DataIntegration.bean.RealTimeTableInfoReq;
import com.jz.dmp.modules.dao.DmpProjectDao;
import com.jz.dmp.modules.dao.DmpRealtimeSyncInfoDao;
import com.jz.dmp.modules.model.*;
import com.jz.dmp.modules.service.DmpRealtimeSyncInfoService;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.regex.Pattern;

/**
 * 实时同步任务(DmpRealtimeSyncInfo)表服务实现类
 *
 * @author Bellamy
 * @since 2021-01-05 14:18:03
 */
@Service("dmpRealtimeSyncInfoService")
@Transactional
public class DmpRealtimeSyncInfoServiceImpl implements DmpRealtimeSyncInfoService {

    @Value("${spring.public-key}")
    private String publicKey;

    @Autowired
    private DmpRealtimeSyncInfoDao dmpRealtimeSyncInfoDao;

    @Autowired
    private DmpProjectDao dmpProjectDao;

    /**
     * 通过ID查询单条数据
     *
     * @param id 主键
     * @return 实例对象
     */
    @Override
    public DmpRealtimeSyncInfo queryById(Integer id) {
        return this.dmpRealtimeSyncInfoDao.queryById(id);
    }

    /**
     * 查询多条数据
     *
     * @param offset 查询起始位置
     * @param limit  查询条数
     * @return 对象列表
     */
    @Override
    public List<DmpRealtimeSyncInfo> queryAllByLimit(int offset, int limit) {
        return this.dmpRealtimeSyncInfoDao.queryAllByLimit(offset, limit);
    }

    /**
     * 新增数据
     *
     * @param dmpRealtimeSyncInfo 实例对象
     * @return 实例对象
     */
    @Override
    public DmpRealtimeSyncInfo insert(DmpRealtimeSyncInfo dmpRealtimeSyncInfo) {
        this.dmpRealtimeSyncInfoDao.insert(dmpRealtimeSyncInfo);
        return dmpRealtimeSyncInfo;
    }

    /**
     * 修改数据
     *
     * @param dmpRealtimeSyncInfo 实例对象
     * @return 实例对象
     */
    @Override
    public DmpRealtimeSyncInfo update(DmpRealtimeSyncInfo dmpRealtimeSyncInfo) {
        this.dmpRealtimeSyncInfoDao.update(dmpRealtimeSyncInfo);
        return this.queryById(dmpRealtimeSyncInfo.getId());
    }

    /**
     * 通过主键删除数据
     *
     * @param id 主键
     * @return 是否成功
     */
    @Override
    @Transactional(rollbackFor = Exception.class, propagation = Propagation.REQUIRES_NEW)
    public boolean deleteById(Integer id) {
        return this.dmpRealtimeSyncInfoDao.deleteById(id) > 0;
    }

    /**
     * 实时同步任务列表分页查询
     *
     * @return
     * @author Bellamy
     */
    @Override
    public PageInfoResponse<RealTimeSyncListDto> queryRealTimeSyncListPage(RealTimeSyncListReq req) throws Exception {
        PageInfoResponse<RealTimeSyncListDto> pageInfoResponse = new PageInfoResponse<>();

        if (StringUtils.isNotEmpty(req.getSourceDatabaseName())) { //来源数据源名称
            req.setSourceDatabaseName(req.getSourceDatabaseName().trim());
        }
        if (StringUtils.isNotEmpty(req.getTargetDatabaseName())) { //目标源数据源名称
            req.setTargetDatabaseName(req.getTargetDatabaseName().trim());
        }
        if (StringUtils.isNotEmpty(req.getTreeId())) {
            //判断是否为整数 是整数返回true,否则返回false
            Pattern pattern = Pattern.compile("^[-\\+]?[\\d]*$");
            if (pattern.matcher(req.getTreeId().trim()).matches()) {
                req.setTreeId(req.getTreeId()); //id
            } else {
                req.setTreeName(req.getTreeId().trim());//节点名称
                req.setTreeId("");
            }
        }

        PageHelper.startPage(req.getPageNum(), req.getPageSize());
        List<RealTimeSyncListDto> list = dmpRealtimeSyncInfoDao.queryRealTimeSyncListPage(req);
        PageInfo<RealTimeSyncListDto> pageInfo = new PageInfo<>(list);

        pageInfoResponse.setCode(ResultCode.SUCCESS);
        pageInfoResponse.setMessage("查询成功");
        pageInfoResponse.setData(pageInfo);
        return pageInfoResponse;
    }

    /**
     * 根据多个实时任务id查询任务信息
     *
     * @return
     * @author Bellamy
     */
    @Override
    public List<DmpRealtimeSyncInfo> queryListById(String[] ids) throws Exception {
        return dmpRealtimeSyncInfoDao.queryListById(ids);
    }

    /**
     * 获取数据源和目标数据源下拉框
     *
     * @return
     * @author Bellamy
     * @since 2021-01-06
     */
    @Override
    public JsonResult<List<DataSourceNameListDto>> queryDatasourceNameList(String projectId, String type) throws Exception {
        List<DataSourceNameListDto> list = dmpRealtimeSyncInfoDao.queryDatasourceNameList(projectId, type);
        return new JsonResult(list);
    }

    /**
     * 新增--选择源表信息
     *
     * @return
     * @author Bellamy
     * @since 2021-01-06
     */
    @Override
    public JsonResult<RealTimeSyncModel> queryTableInfoByParams(RealTimeTableInfoReq req) throws Exception {
        Map<String, Map<String, String>> paramsMap = null;
        //根据来源数据源id获取数据信息
        RealTimeSyncDataSourceModel sourceDbInfo = dmpRealtimeSyncInfoDao.querygSourceDbInfoById(req.getSrcDatasourceId());
        sourceDbInfo.setPassword(new BaseService().decode(sourceDbInfo.getPassword(), publicKey));

        Map map = new HashMap();
        map.put("srcDatasourceId", req.getSrcDatasourceId());
        map.put("targetDatasourceId", req.getTargetDatasourceId());
        //数据源对应的表详细信息  判断表的记录在同步任务中是否存在记录,如果已经存在,说明同步过
        List<Map> list = dmpRealtimeSyncInfoDao.queryRealTimeInfoByDataSourceId(map);
        if (list.size() > 0 && list != null) {
            paramsMap = new HashMap<>();
            for (Map item : list) {
                String srcTableName = item.get("srcTableName").toString();
                paramsMap.put(srcTableName, item);
            }
        }
        Map<String, String> blackMapSetting = null;
        //查询来源数据源的黑名单表
        Map blackTableMap = dmpRealtimeSyncInfoDao.queryBlackTableByDataSourceId(req.getSrcDatasourceId());
        if (blackTableMap.size() > 0 && blackTableMap != null) {
            String blacklistTable = blackTableMap.get("blacklistTable").toString();
            if (StringUtils.isNotEmpty(blacklistTable)) {
                String[] blacklistTableArr = blacklistTable.split(",");
                if (blacklistTableArr.length > 0 && blacklistTableArr != null) {
                    blackMapSetting = new HashMap<>();
                    for (String str : blacklistTableArr) {
                        blackMapSetting.put(str, str);
                    }
                }
            }
        }
        //来源数据源下的表信息
        List<RealTimeSyncTableModel> dataSourceTables = DBUtil.getDataSourceTables(sourceDbInfo, req.getToQueryTableName(), true, paramsMap, req.getBlacklistTables(), req.getSelectTablesName(), blackMapSetting);

        RealTimeSyncModel realTimeSyncModel = new RealTimeSyncModel();
        realTimeSyncModel.setSrcdatasourceId(Integer.valueOf(req.getSrcDatasourceId()));
        realTimeSyncModel.setDataSourceName(sourceDbInfo.getDatasourceName());
        realTimeSyncModel.setDbName(sourceDbInfo.getDbName());
        realTimeSyncModel.setTables(dataSourceTables); //来源数据源下的表信息
        return new JsonResult(realTimeSyncModel);
    }

    /**
     * 新增--选择目标信息
     *
     * @return
     * @author Bellamy
     * @since 2021-01-07
     */
    @Override
    public JsonResult selectTargetDatasourceInfo(String projectId) throws Exception {
        DmpProjectSystemInfo dmpProjectSystemInfo = dmpProjectDao.queryProjectSystemInfo(Long.valueOf(projectId));
        String kafkaConnectUrl = dmpProjectSystemInfo.getKafkaConnectorUrl(); //kafka 连接信息
        String[] arr = kafkaConnectUrl.split(",");
        return new JsonResult(arr);
    }

    /**
     * 设置成黑名单时 ,检查是否有同步任务存在运行
     *
     * @return
     * @author Bellamy
     * @since 2021-01-07
     */
    @Override
    public JsonResult queryRealTimeInfoByDataSourceId(String srcDatasourceId, String sourceTableName) throws Exception {
        Map map = new HashMap();
        map.put("srcDatasourceId", srcDatasourceId); //来源表 数据源id
        map.put("sourceTableName", sourceTableName); // 来源表
        List<Map> list = dmpRealtimeSyncInfoDao.queryRealTimeInfoByDataSourceId(map);
        boolean flag = false;
        if (list.size() > 0 && list != null) {
            flag = true;
        }
        return new JsonResult(flag);
    }
}